package com.coherentlogic.fred.client.core.domain;

import static com.coherentlogic.fred.client.core.util.Constants.DEFAULT_LIMIT;
import static com.coherentlogic.fred.client.core.util.Constants.COUNT;
import static com.coherentlogic.fred.client.core.util.Constants.LIMIT;
import static com.coherentlogic.fred.client.core.util.Constants.OFFSET;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY_VALUE;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_END;
import static com.coherentlogic.fred.client.core.util.Constants.REALTIME_START;
import static com.coherentlogic.fred.client.core.util.Constants.SORT_ORDER;
import static com.coherentlogic.fred.client.core.util.Constants.ORDER_BY;
import static com.coherentlogic.fred.client.core.util.Constants.TAGS;

import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import com.coherentlogic.coherent.data.model.core.domain.DefaultObject;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * FRED tags are attributes assigned to series. See the related request
 * <a href="http://api.stlouisfed.org/docs/fred/related_tags.html">
 * fred/related_tags</a>. 
 *
 * @todo Consider creating a CountableSpecification as the count property is
 *  repeated in a few places.
 *
 * @author <a href="mailto:support@coherentlogic.com">Support</a>
 */
@Entity
@Table(name=TAGS)
@XStreamAlias(TAGS)
public class Tags extends DefaultObject
    implements
        RealtimeBoundSpecification,
        PaginationSpecification,
        OrderBySpecification,
        SortOrderSpecification {

    private static final long serialVersionUID = -8240224708430342047L;

    @XStreamAlias(COUNT)
    @XStreamAsAttribute
    private int count = 0;

    @XStreamImplicit
    private List<Tag> tagList = null;

    @XStreamAlias(REALTIME_START)
    @XStreamAsAttribute
    private Date realtimeStart = null;

    @XStreamAlias(REALTIME_END)
    @XStreamAsAttribute
    private Date realtimeEnd = null;

    @XStreamAlias(SORT_ORDER)
    @XStreamAsAttribute
    private SortOrder sortOrder = SortOrder.asc;

    @XStreamAlias(ORDER_BY)
    @XStreamAsAttribute
    private OrderBy orderBy = OrderBy.seriesId;

    /**
     * @todo Make this value a constant.
     */
    @XStreamAlias(LIMIT)
    @XStreamAsAttribute
    private long limit = DEFAULT_LIMIT;

    @XStreamAlias(OFFSET)
    @XStreamAsAttribute
    private int offset = 0;

    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }

    @OneToMany(cascade=CascadeType.ALL)
    public List<Tag> getTagList() {
        return tagList;
    }

    public void setTagList(List<Tag> tagList) {
        this.tagList = tagList;
    }

    @Override
    public Date getRealtimeStart() {
        return realtimeStart;
    }

    @Override
    public void setRealtimeStart(Date realtimeStart) {
        this.realtimeStart = realtimeStart;
    }

    @Override
    public Date getRealtimeEnd() {
        return realtimeEnd;
    }

    @Override
    public void setRealtimeEnd(Date realtimeEnd) {
        this.realtimeEnd = realtimeEnd;
    }

    @Override
    public SortOrder getSortOrder() {
        return sortOrder;
    }

    @Override
    public void setSortOrder(SortOrder sortOrder) {
        this.sortOrder = sortOrder;
    }

    @Column(name=ORDER_BY_VALUE)
    @Override
    public OrderBy getOrderBy() {
        return orderBy;
    }

    @Override
    public void setOrderBy(OrderBy orderBy) {
        this.orderBy = orderBy;
    }

    public long getLimit() {
        return limit;
    }

    public void setLimit(long limit) {
        this.limit = limit;
    }

    public int getOffset() {
        return offset;
    }

    public void setOffset(int offset) {
        this.offset = offset;
    }
}
